package io.zipcoder.tc_spring_poll_application.controller;

import io.zipcoder.tc_spring_poll_application.domain.Poll;
import io.zipcoder.tc_spring_poll_application.exceptions.ResourceNotFoundException;
import io.zipcoder.tc_spring_poll_application.repositories.PollRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.support.ServletUriComponentsBuilder;

@RestController
public class PollController {
    private PollRepository pollRepository;
    private Poll poll;

    @Autowired
    public PollController(PollRepository pollRepository) {
        this.pollRepository = pollRepository;
    }

    @RequestMapping(value="/polls", method= RequestMethod.GET)
    public ResponseEntity<Iterable<Poll>> getAllPolls(){
        Iterable<Poll> allPolls = pollRepository.findAll();
        return new ResponseEntity<>(allPolls, HttpStatus.OK);
    }

    @RequestMapping(value="/polls", method= RequestMethod.POST)
    public ResponseEntity<?> createPoll(@RequestBody Poll poll){
        poll = pollRepository.save(poll);
        HttpHeaders headers = new HttpHeaders();
        headers.setLocation( ServletUriComponentsBuilder
                        .fromCurrentRequest()
                        .path("/{id}")
                        .buildAndExpand(poll.getId())
                        .toUri());
        return new ResponseEntity<>(null, HttpStatus.CREATED);
    }

    //I need to go in and modify this createPoll method so that it does something with the URI from part 3.1.4

    @RequestMapping(value = "/polls/{pollId}", method = RequestMethod.GET)
    public ResponseEntity<?> getPoll(@PathVariable Long pollId){

        verifyPoll(pollId);
        //verifies that the pollId exist in the repository

        Poll p = pollRepository.findOne(pollId);
        return new ResponseEntity<>(p,HttpStatus.OK);
    }

    //updates the poll
    @RequestMapping(value = "/polls/{pollId}", method = RequestMethod.PUT)
    public ResponseEntity<?> updatePoll(@RequestBody Poll poll, @PathVariable Long pollId){
        // Save the entity

        verifyPoll(pollId);
        //verifies that the pollId exist in the repository


        Poll p = pollRepository.save(poll);
        return new ResponseEntity<>(HttpStatus.OK);
    }


    //Deletes the poll
    @RequestMapping(value = "/polls/{pollId}", method = RequestMethod.DELETE)
    public ResponseEntity<?> deletePoll(@PathVariable Long pollId){

        verifyPoll(pollId);
        //verifies that the pollId exist in the repository


        pollRepository.delete(pollId);
        return new ResponseEntity<>(HttpStatus.OK);
    }

    public void verifyPoll(Long pollId){

//        this.poll.setId(pollId);

        try{
            if(pollRepository.findOne(pollId) == null){
                throw new ResourceNotFoundException();
            }
            else{
                System.out.println("It Works.");

            }
        }catch (ResourceNotFoundException e){
            System.out.println("Resource Not Found");
        }
    }
}
